@@ -132,21 +132,26 @@ editGenome = function(pop,ind,chr,segSites,allele,
     sel = chr==selChr
     selSegSites = segSites[sel]
     selAllele = allele[sel]
-    pop@geno[[selChr]][selSegSites,,ind] = selAllele
-  }
-  pop@gxe = vector("list",simParam$nTraits)
-  pop@gv = matrix(NA_real_,nrow=pop@nInd,
-                  ncol=simParam$nTraits)
-  if(simParam$nTraits>=1){
-    for(i in 1:simParam$nTraits){
-      tmp = getGv(simParam$traits[[i]],pop,simParam$nThreads)
-      pop@gv[,i] = tmp[[1]]
-      if(length(tmp)>1){
-        pop@gxe[[i]] = tmp[[2]]
+    for(selSite in selSegSites){
+      BYTE = (selSite-1L)%/%8L + 1L
+      BIT = (selSite-1L)%%8L + 1L
+      for(selInd in ind){
+        for(i in 1:pop@ploidy){
+          TMP = pop@geno[[selChr]][BYTE,i,selInd]
+          TMP = rawToBits(TMP)
+          TMP[BIT] = selAllele
+          TMP = packBits(TMP)
+          pop@geno[[selChr]][BYTE,i,selInd] = TMP
+        }
       }
     }
   }
-  validObject(pop)
+  # Reset population
+  PHENO = pop@pheno
+  EBV = pop@ebv
+  pop = resetPop(pop=pop, simParam=simParam)
+  pop@pheno = PHENO
+  pop@ebv = EBV
   return(pop)
 }
 
@@ -266,7 +271,6 @@ editGenomeTopQtl = function(pop, ind, nQtl, trait = 1, increase = TRUE, simParam
                      allele = targetQtl[[4]],
                      simParam = simParam)
   }
-  validObject(pop)
   return(pop)
 }
 
@@ -313,9 +317,9 @@ usefulness = function(pop,trait=1,use="gv",p=0.1,
   if(is.null(simParam)){
     simParam = get("SP",envir=.GlobalEnv)
   }
-  response = getResponse(pop=pop,trait=trait,use=use,
-                         simParam=simParam,...)
-  response = sort(response,decreasing=selectTop)
+  response = getResponse(pop=pop, trait=trait, use=use,
+                         simParam=simParam, ...)
+  response = sort(response, decreasing=selectTop)
   response = response[1:ceiling(p*length(response))]
   return(mean(response))
 }
@@ -394,4 +398,116 @@ rotMat = function(X){
   ans = svd(X)
   u = t(ans$u)*sqrt(pmax(ans$d,0))
   return(t(ans$v%*%u))
-}
\ No newline at end of file
+}
+
+#' @title Add Random Mutations
+#' 
+#' @description
+#' Adds random mutations to individuals in a 
+#' population. Note that any existing phenotypes 
+#' or EBVs are kept. Thus, the user will need to run 
+#' \code{\link{setPheno}} and/or \code{\link{setEBV}} 
+#' to generate new phenotypes or EBVs that reflect 
+#' changes introduced by the new mutations.
+#' 
+#' @param pop an object of \code{\link{Pop-class}}
+#' @param mutRate rate of new mutations
+#' @param returnPos should the positions of mutations be returned
+#' @param simParam an object of \code{\link{SimParam}}
+#'
+#' @return an object of \code{\link{Pop-class}} if 
+#' returnPos=FALSE or a list containing a 
+#' \code{\link{Pop-class}} and a data.frame containing the 
+#' postions of mutations if returnPos=TRUE
+#' 
+#' @examples 
+#' #Create founder haplotypes
+#' founderPop = quickHaplo(nInd=2, nChr=1, segSites=10)
+#' 
+#' #Set simulation parameters
+#' SP = SimParam$new(founderPop)
+#' SP$addTraitA(10)
+#' 
+#' #Create population
+#' pop = newPop(founderPop, simParam=SP)
+#' 
+#' #Introduce mutations
+#' pop = mutate(pop, simParam=SP)
+#' 
+#' @export
+mutate = function(pop, mutRate=2.5e-8, returnPos=FALSE, simParam=NULL){
+
+  # Mutation history variable
+  IND=NULL; CHR=NULL; HAP=NULL; SITE=NULL
+  
+  # Number of haplotypes per chromosome
+  nHap = pop@nInd*pop@ploidy
+  
+  # Number of total sites
+  s = sum(pop@nLoci)
+  
+  # Number of mutations per haplotype
+  nMut = rbinom(nHap, s, mutRate)
+  
+  if(any(nMut>0L)){
+    for(take in which(nMut>0L)){
+      # Determine haplotype and individual
+      ind = (take-1L)%/%pop@ploidy + 1L
+      hap = (take-1L)%%pop@ploidy + 1L
+      
+      # Sample mutation sites
+      sites = sampleInt(nMut[take], s) + 1L
+      
+      # Resolve all mutations
+      chr = 1L
+      for(i in sites){
+        # Find chromosome
+        repeat{
+          if(i > sum(pop@nLoci[1L:chr])){
+            chr = chr + 1L
+          }else{
+            break
+          }
+        }
+        
+        # Find site
+        if(chr>1L){
+          site = i - sum(pop@nLoci[1L:(chr-1L)]) 
+        }else{
+          site = i
+        }
+        
+        # Create mutation
+        BYTE = (site-1L)%/%8L + 1L
+        BIT = (site-1L)%%8L + 1L
+        TMP = pop@geno[[chr]][BYTE,hap,ind]
+        TMP = rawToBits(TMP)
+        TMP[BIT] = ifelse(TMP[BIT], as.raw(0L), as.raw(1L))
+        TMP = packBits(TMP)
+        pop@geno[[chr]][BYTE,hap,ind] = TMP
+        
+        # Record results
+        if(returnPos){
+          IND = c(IND, ind)
+          CHR = c(CHR, chr)
+          HAP = c(HAP, hap)
+          SITE = c(SITE, site)
+        }
+      }
+    }
+    
+    # Reset population
+    PHENO = pop@pheno
+    EBV = pop@ebv
+    pop = resetPop(pop=pop, simParam=simParam)
+    pop@pheno = PHENO
+    pop@ebv = EBV
+  }
+  
+  # Return results
+  if(returnPos){
+    return(list(pop,data.frame(individual=IND,chromosome=CHR,haplotype=HAP,site=SITE)))
+  }else{
+    return(pop)
+  }
+}
